Attribute VB_Name = "modGlobal"
Option Explicit

' Used for background bitmap dialog
Global gnBackBmp As Integer

' Used to keep track of online title info
Global gszOnlineService As String
Global gszOnlineTitle As String
Global gbOnlineCancel As Boolean

' Used for History and Back lists
Global Const MAX_HISTORY = 50
Global Const MAX_BACK = 50
Global gszTopicTitle As String
Global glTopicIndex As Long
Global glTopicVA As Long
Global glTopicScroll As Long

' Used to keep the last Search dialog values
Global gszSearch As String
Global gnNear As Integer
Global gnAnd As Integer

' Used to keep track of which search highlight is current
Global glHighlightIndex As Long

' Rect structure used to keep track of the secondary window's size and pos
Type vbRect
    Left As Long
    Top As Long
    Width As Long
    Height As Long
End Type

Global gRectSec As vbRect
    
' Rect structure used to get a window's client rectangle
Type cRect
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

' Used in Hotspot_Clicked event (type of hotspot that was clicked)
Global Const HOTSPOT_STRING = 0
Global Const HOTSPOT_HASH = 1
Global Const HOTSPOT_POPUPHASH = 2
Global Const HOTSPOT_UNKNOWN = 3

' These constants are used with the lMVTitleGetInfo() MediaView API function
Global Const TTLINF_TITLE = 1
Global Const TTLINF_GROUPS = 13
Global Const TTLINF_KEYINDEX = 14
Global Const TTLINF_HASINDEX = 100
Global Const TTLINF_TOPICADDR = 101
Global Const TTLINF_TOPICTITLE = 102
Global Const TTLINF_ADDRTITLE = 110

' MediaView APIs used in this sample
Declare Function vaMVConvertAddr Lib "mvcl14n.dll" (ByVal hTitle As Long, ByVal Addr As Long) As Long
Declare Function lMVTitleGetInfo Lib "mvcl14n.dll" (ByVal hTitle As Long, ByVal nInfoType As Integer, ByVal lParam1 As Long, ByVal lParam2 As String) As Long
Declare Function hMVWordWheelOpenTitle Lib "mvcl14n.dll" (ByVal hTitle As Long, ByVal wwFile As String) As Long
Declare Function lMVWordWheelLength Lib "mvcl14n.dll" (ByVal hWordWheel As Long) As Long
Declare Function nMVWordWheelLookup Lib "mvcl14n.dll" (ByVal hWordWheel As Long, ByVal lIndex As Long, ByVal szKeyword As String, ByVal cbKeyword As Integer) As Integer
Declare Function nMVKeyIndexGetCount Lib "mvcl14n.dll" (ByVal hWordWheel As Long, ByVal szKeyword As String) As Long
Declare Function lMVKeyIndexGetAddrs Lib "mvcl14n.dll" (ByVal hWordWheel As Long, ByVal szKeyword As String, ByVal nStart As Integer, ByVal nCount As Integer, lpAddr As Long) As Long
Declare Function lMVWordWheelPrefix Lib "mvcl14n.dll" (ByVal hWordWheel As Long, ByVal szPrefix As String) As Long

' The "Group" structure holds the information for each group in the "Groups()" array
Type Group
    Filename As String
    Title As String
End Type

Global Groups() As Group

' The "Index" structure holds the information for each index in the "Indexes()" array
Type Index
    Filename As String * 1
    Title As String
End Type

Global Indexes() As Index
 
' Windows 32-Bit APIs used
Declare Function lstrcpy Lib "kernel32" Alias "lstrcpyA" (ByVal lpString1 As String, ByVal lpString2 As Long) As Long
Declare Function GetClientRect Lib "user32" (ByVal hWnd As Long, lpRect As cRect) As Long

Public Sub CenterForm(frm As Form, bScreen As Boolean)

    ' This sub centers a form (1st argument), either on the screen or on the app's main form.
    ' If bScreen is True, the form is centered on the screen.  If bScreen is False, and the
    ' form IS the main app's form, the main app's form is centered on the screen anyway.
    ' If bScreen is False, and the form is not the app's main form, the form is centered on
    ' the app's main form.
    
    ' If the flag is set ...
    If bScreen Then
        ' Center the form on the screen
        frm.Move (Screen.Width - frm.Width) / 2, (Screen.Height - frm.Height) / 2, _
        frm.Width, frm.Height
    
    Else
        ' If this is the main form ...
        If frm Is frmMain Then
            ' Center the app's main form on the screen
            frm.Move (Screen.Width - frm.Width) / 2, (Screen.Height - frm.Height) / 2, _
            frm.Width, frm.Height
        Else
            ' Center this form on the app's main form
            frm.Move frmMain.Left + ((frmMain.Width - frm.Width) / 2), _
            frmMain.Top + ((frmMain.Height - frm.Height) / 2), frm.Width, frm.Height
        End If
    
    End If

End Sub

Public Sub StoreTopicInfo()
    
    ' By the time we reach the Topic_Changing event, the MediaViewTopic,
    ' TopicIndex, TopicVA, and TopicScroll will have changed.
    ' So we store these values each time we are about to make a jump so
    ' that we can correctly add to the Back and History lists at that time.
    
    gszTopicTitle = frmMain.mvMain.MediaViewTopic
    glTopicIndex = frmMain.mvMain.TopicIndex
    glTopicVA = frmMain.mvMain.TopicVA
    glTopicScroll = frmMain.mvMain.TopicScroll

End Sub

Public Function MakeLong(LOWORD As Long, HIWORD As Long) As Long
    
    ' This simply combines two longs (WORDs) into one long (DWORD)
    MakeLong = HIWORD * &H10000 Or LOWORD

End Function

Public Sub OutputGroups()

    ' Used only for debugging
    
    Dim i As Integer
    
    For i = 0 To (UBound(Groups) - 1)
        Debug.Print Groups(i).Filename
        Debug.Print Groups(i).Title
    Next i

End Sub

Public Sub LoadGroups()
    
    Dim nGroupIndex As Long         ' index counter for all groups in the new title
    Dim lpBuffer As String          ' string containing the group information
    Dim lpFilename As String        ' temporary string to hold the group filename
    Dim lpTitle As String           ' temporary string to hold the group title
    Dim nSGIndex As Integer         ' index counter for the searchable groups
    Dim nPos As Integer             ' position in lpBuffer used for parsing
    Dim nStart As Integer           ' position in lpBuffer used for parsing
    Dim nStop As Integer            ' position in lpBuffer used for parsing
    Dim bGroupLoadFailed As Boolean ' flag indicating whether a group load failed
    
    ' Init variables
    bGroupLoadFailed = False
    nGroupIndex = 0
    nSGIndex = 0
    lpBuffer = String(257, 0)
    
    ' Clear the array
    ReDim Groups(nSGIndex)
    
    ' We're interested in searchable groups which contain topics.  Searchable
    ' groups were defined with a group title.  Not all searchable groups contain
    ' topics.  So if there's a title in the group string, AND there are topics
    ' in that group, then add the group info to the Groups() array.
    
    ' Loop through all groups
    Do Until (lMVTitleGetInfo(frmMain.mvMain.hTitle, TTLINF_GROUPS, MakeLong(Len(lpBuffer), nGroupIndex), lpBuffer)) = -1
        ' First check if there's a title in the group string (text inside quotes)
        nStart = InStr(lpBuffer, Chr$(34)) + 1
        nStop = InStr(nStart, lpBuffer, Chr$(34))
        ' The following prevents an error if nStart is greater than nStop
        If nStart > nStop Then nStop = nStart
        lpTitle = Mid(lpBuffer, nStart, nStop - nStart)
        If lpTitle <> "" Then
            ' Get the group filename
            nPos = InStr(lpBuffer, ".")
            lpFilename = Left(lpBuffer, nPos - 1)
            ' Set an error trap to look for a group load failure error
            On Error GoTo ErrorHandler
            ' Add the topics for the current group to the topic list
            frmMain.mvMain.AddGroupToTopicList lpFilename
            ' Reset error trapping
            On Error GoTo 0
            ' If an attempt to load the group failed, then don't add it to the array
            If Not bGroupLoadFailed Then
                ' Increase the size of the array to fit the new entry
                ReDim Preserve Groups(nSGIndex + 1)
                ' Store the group filename and title (which we've already found)
                Groups(nSGIndex).Filename = lpFilename
                Groups(nSGIndex).Title = lpTitle
                ' Add one to the searchable group index
                nSGIndex = nSGIndex + 1
            Else
                ' Reset the flag for the next group
                bGroupLoadFailed = False
            End If
        End If
        ' Check the next group
        nGroupIndex = nGroupIndex + 1
    Loop
    
    Exit Sub

ErrorHandler:
    
    If Err = 29006 Then
        bGroupLoadFailed = True
        Resume Next
    Else
        Call MsgBox("Error " & Err & " occurred." & Chr$(13) & Chr$(10) _
                & Error(Err), vbExclamation, "Error")
    End If

End Sub

Public Sub LoadIndexes()
    
    Dim nKeyIndex As Long           ' index for all keyword indexes in the title
    Dim lpBuffer As String          ' string containing the keyword index info
    Dim lpTitle As String           ' temporary string to hold the index title
    Dim nPos As Integer             ' position in lpBuffer used for parsing
    Dim lKeyChar As Long            ' contains the key index filename (as a long)
    
    ' Init variables
    nKeyIndex = 0
    lpBuffer = String(257, 0)
    
    ' Clear the array
    ReDim Indexes(nKeyIndex)
    
    ' In order for a keyword index to be added to the .M14 title file, it must be
    ' defined in the .MVP file with an index title (even if it's ""), and the RTF
    ' source must contain at least one keyword 'K' footnote which references the
    ' index.  So if the following call succeeds, we know we have a keyword index
    ' which contains at least one keyword, so add it's info to the Indexes() array.
    
    ' Get the first keyword index (if any)
    lKeyChar = lMVTitleGetInfo(frmMain.mvMain.hTitle, TTLINF_KEYINDEX, MakeLong(Len(lpBuffer), nKeyIndex), lpBuffer)
    
    ' Loop through all indexes
    Do Until lKeyChar = 0
        ' First check if there's a title in the index string (text other than null)
        nPos = InStr(lpBuffer, Chr$(0))
        ' If the title was blank, fill it in with a default one
        If nPos = 1 Then
            lpTitle = "<Untitled Index>"
        Else
            lpTitle = Mid(lpBuffer, 1, nPos - 1)
        End If
        ' Increase the size of the array to fit the new entry
        ReDim Preserve Indexes(nKeyIndex + 1)
        ' Store the index filename.  It is the character equivalent
        ' of the long value returned in the lMVTitleGetInfo() call.
        Indexes(nKeyIndex).Filename = Chr$(lKeyChar)
        ' Store the index title (which we've already found)
        Indexes(nKeyIndex).Title = lpTitle
        ' Check the next index
        nKeyIndex = nKeyIndex + 1
        lKeyChar = lMVTitleGetInfo(frmMain.mvMain.hTitle, TTLINF_KEYINDEX, MakeLong(Len(lpBuffer), nKeyIndex), lpBuffer)
    Loop
    
    Exit Sub

End Sub

Public Sub OutputIndexes()

    ' Used only for debugging
    
    Dim i As Integer
    
    For i = 0 To (UBound(Indexes) - 1)
        Debug.Print Indexes(i).Filename
        Debug.Print Indexes(i).Title
    Next i

End Sub

Public Sub SecondaryJump(szContextString As String)

    ' Load the secondary topic window, but don't show it yet
    Load frmSecondary
    
    ' Jump to the desired topic
    frmSecondary.mvSecondary.SetTopicFromContextString szContextString
    
    ' Set the window caption to the main mv topic title
    frmSecondary.Caption = frmMain.mvMain.MediaViewTopic
    
    ' Now show the form
    frmSecondary.Show

End Sub
